<?php 
/**
 * Plugin Name: Disable Url Decoding (Matomo Plugin)
 * Plugin URI: http://plugins.matomo.org/DisableUrlDecoding
 * Description: Disables URL-Decoding of current location and referer to avoid losing data in URL-encoded paths and parameters.
 * Author: Anisatum
 * Author URI: https://github.com/Anisatum/matomo-plugin-DisableUrlDecoding/
 * Version: 1.0.1
 */
?><?php
/**
 * Matomo - free/libre analytics platform
 *
 * @link https://matomo.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html GPL v3 or later
 */

namespace Piwik\Plugins\DisableUrlDecoding;

use Piwik\Piwik;

 
if (defined( 'ABSPATH')
&& function_exists('add_action')) {
    $path = '/matomo/app/core/Plugin.php';
    if (defined('WP_PLUGIN_DIR') && WP_PLUGIN_DIR && file_exists(WP_PLUGIN_DIR . $path)) {
        require_once WP_PLUGIN_DIR . $path;
    } elseif (defined('WPMU_PLUGIN_DIR') && WPMU_PLUGIN_DIR && file_exists(WPMU_PLUGIN_DIR . $path)) {
        require_once WPMU_PLUGIN_DIR . $path;
    } else {
        return;
    }
    add_action('plugins_loaded', function () {
        if (function_exists('matomo_add_plugin')) {
            matomo_add_plugin(__DIR__, __FILE__, true);
        }
    });
}

class DisableUrlDecoding extends \Piwik\Plugin
{
    /**
     * @see \Piwik\Plugin::registerEvents
     */
    public function registerEvents()
    {
        return [
            'CustomJsTracker.manipulateJsTracker' => 'manipulateJsTracker',
            'SystemSettings.updated' => 'updated'
        ];
    }

    public function isTrackerPlugin()
    {
        return true;
    }

    public function updated(SystemSettings $settings)
    {
        if ($settings->getPluginName() === 'DisableUrlDecoding') {
            Piwik::postEvent('CustomJsTracker.updateTracker');
        }
    }

    public function manipulateJsTracker(&$content)
    {
        $systemSettings = new SystemSettings();
        $jsSettings = [];

        if ($systemSettings->disableFor->getValue())
            $jsSettings['disableFor'] = $systemSettings->disableFor->getValue();
        if ($jsSettings['disableFor'] == SystemSettings::TYPE_REGEXP && $systemSettings->disableRegexp->getValue())
            $jsSettings['regexp'] = $systemSettings->disableRegexp->getValue();
        if ($jsSettings['disableFor'] == SystemSettings::TYPE_PARAMS && $systemSettings->disableParams->getValue())
            $jsSettings['params'] = $systemSettings->disableParams->getValue();


        $jsSettings = json_encode($jsSettings);
        $content = preg_replace('/Matomo.DisableUrlDecoding\s*\=\s*\{\}\;/i',
            "Matomo.DisableUrlDecoding=$jsSettings;",
            $content
        );
    }
}
